from abc import ABCMeta, abstractmethod, abstractproperty


class Key(object):
    """
    Decorator class to specify modifier key relations.
    """

    def __init__(self, hex_key_code):
        self.code = hex_key_code
        self.children = None

    def modify(self, *args):
        """
        Specifies Keys that will be modified by a current key.

        :param args: Keys to be modified by current key.
        :rtype: Key
        :return: New instance of Key with children Keys to be modified.
        """
        for arg in args:
            if not isinstance(arg, Key):
                raise Exception('Key instance is expected.')

        modified_key_press = Key(self.code)
        modified_key_press.children = args
        return modified_key_press


class IKeyboard(object):

    __metaclass__ = ABCMeta

    @abstractproperty
    def codes(self):
        """
        Container class to store KeyCodes as Keys class.
        """

    @abstractmethod
    def press_key(self, hex_key_code):
        """
        Presses (and releases) key specified by a hex code.

        :param int hex_key_code: integer value holding hexadecimal code for 
        a key to be pressed.
        """

    @abstractmethod
    def press_key_and_hold(self, hex_key_code):
        """
        Presses (and holds) key specified by a hex code.

        :param int hex_key_code: integer value holding hexadecimal code for 
        a key to be pressed.
        """

    @abstractmethod
    def release_key(self, hex_key_code):
        """
        Releases key specified by a hex code.

        :param int hex_key_code: integer value holding hexadecimal code for 
        a key to be released.
        """

    @abstractmethod
    def send(self, *args, **kwargs):
        """
        Send key events as specified by Keys.

        If Key contains children Keys they will be recursively
        processed with current Key code pressed as a modifier key.

        :param args: Keys to send.
        :param kwargs: "delay" between keys in seconds.
        """



from makima.windows.ui_element import WinUIElement, _IUIAutomation


def initialize_app_ref_for_win(name, class_name=None):
    for window in WinUIElement(_IUIAutomation.GetRootElement()).get_acc_children_elements():
        if class_name is not None:
            if window.get_acc_name == name and window.get_class_name == class_name:
                return window
        else:
            if window.get_acc_name == name:
                return window
    return None


def GetRootElement():
    return WinUIElement(_IUIAutomation.GetRootElement())


def find_windows(name):
    for window in WinUIElement(_IUIAutomation.GetRootElement()).get_acc_children_elements():
        if window.get_acc_name == name:
            return window
    return None



class Stack(object):

    """Stack"""

    def __init__(self):
        self.items = []

    def is_empty(self):
        """is null"""
        return self.items == []

    def is_not_empty(self):
        """is null"""
        return self.items != []

    def push(self, item):
        """add item"""
        self.items.append(item)

    def pop(self):
        """pop item"""
        return self.items.pop()

    def reverse(self):
        """reverse item"""
        return self.items.reverse()

    def peek(self):
        """pop item"""
        return self.items[len(self.items) - 1]

    def size(self):
        """return sie of stack"""
        return len(self.items)



import ctypes.wintypes

import comtypes
from comtypes import GUID, IUnknown, COMMETHOD, POINTER, helpstring

HRESULT = ctypes.wintypes.DWORD
comtypes.client.GetModule('oleacc.dll')
comtypes.client.GetModule('uiautomationcore.dll')

class IAccessibleEx(IUnknown):
    _case_insensitive_ = True
    _iid_ = GUID('{F8b80AdA-2C44-48D0-89BE-5FF23C9CD875}')
    _idlflags_ = []


_methods_ = [
    COMMETHOD(
        [helpstring('Method GetObjectForChild')],
        HRESULT,
        'GetObjectForChild',
        (['in'], ctypes.c_ulong, 'idChild'),
        (
            ['retval', 'out'],
            POINTER(POINTER(IAccessibleEx)),
            'pRetVal'
        ),
    ),
    COMMETHOD(
        [helpstring('Method GetIAccessiblePair')],
        HRESULT,
        'GetIAccessiblePair',
        (['out'], POINTER(POINTER(comtypes.gen.Accessibility.IAccessible)), 'ppAcc'),
        (['out'], ctypes.c_ulong, 'pidChild'),
    ),
    COMMETHOD(
        [helpstring('Method GetRuntimeId')],
        HRESULT,
        'GetRuntimeId',
        (['out', 'retval'], ctypes.c_int, 'pRetVal'),
    ),
    COMMETHOD(
        [helpstring('Method ConvertReturnedElement')],
        HRESULT,
        'ConvertReturnedElement',
        (['in'], POINTER(comtypes.gen.UIAutomationClient.IRawElementProviderSimple), 'pIn'),
        (
            ['out'],
            POINTER(POINTER(IAccessibleEx)),
            'ppRetValOut'
        ),
    ),
]



import numpy as np
import cv2
import pyautogui
from matplotlib import pyplot as plt
from sklearn.neighbors import LocalOutlierFactor


# img = pyautogui.screenshot()
# print(img)
# img1 = np.array(img)
img1 = cv2.imread('C:\\Users\\hanhuang\\1.png', 0)  # trainImage
img2 = cv2.imread('C:\\Users\\hanhuang\\2.png', 0)  # trainImage


img1 = img1[100:900, 50:1850]
img2 = img2[100:900, 50:1850]

# image = img1[100:900, 50:1850]
# plt.imshow(image)
# plt.show()
# Initiate SIFT detector
sift = cv2.SIFT_create()

# find the keypoints and descriptors with SIFT
kp1, des1 = sift.detectAndCompute(img1, None)
kp2, des2 = sift.detectAndCompute(img2, None)

# FLANN parameters
FLANN_INDEX_KDTREE = 0
index_params = dict(algorithm=FLANN_INDEX_KDTREE, trees=5)
search_params = dict(checks=50)  # or pass empty dictionary

flann = cv2.FlannBasedMatcher(index_params, search_params)

# 使用基于FLANN的匹配器, 筛选符合条件的坐标
matches = flann.knnMatch(des1, des2, k=2)

# Need to draw only good matches, so create a mask
matchesMask = [[0, 0] for i in range(len(matches))]

result = []
good = []

# ratio test as per Lowe's paper
for i, (m, n) in enumerate(matches):
    # distance越小匹配度越高
    if m.distance < 0.4 * n.distance:
        good.append(m)
        result.append(kp1[m.queryIdx].pt)
        matchesMask[i] = [1, 0]

x_of_point1_of_transform_image = 0
y_of_point1_of_transform_image = 0
x_of_point3_of_transform_image = 0
y_of_point3_of_transform_image = 0
x = 0
y = 0

MIN_MATCH_COUNT = 10
if len(good) > MIN_MATCH_COUNT:
    # Gets the coordinates of the key points
    src_pts = np.float32([kp1[m.queryIdx].pt for m in good]).reshape(-1, 1, 2)
    dst_pts = np.float32([kp2[m.trainIdx].pt for m in good]).reshape(-1, 1, 2)
    # Compute the transformation matrix and MASK
    M, mask = cv2.findHomography(src_pts, dst_pts, cv2.RANSAC, 5.0)
    h, w = img1.shape

    # Calculate the original image size
    original_image_size = h * w

    # Use the transformation matrix to obtain the coordinates of the four corners of the original image after
    # transformation
    pts = np.float32([[0, 0], [0, h - 1], [w - 1, h - 1], [w - 1, 0]]).reshape(-1, 1, 2)
    dst = cv2.perspectiveTransform(pts, M)

    # Get the coordinates of the lower right corner of the transformed image
    x_of_point1_of_transform_image = [np.int32(dst)][0][0][0][0]
    y_of_point1_of_transform_image = [np.int32(dst)][0][0][0][1]

    # Get the coordinates of the lower right corner of the transformed image
    x_of_point3_of_transform_image = [np.int32(dst)][0][2][0][0]
    y_of_point3_of_transform_image = [np.int32(dst)][0][2][0][1]

    print(x_of_point1_of_transform_image, y_of_point1_of_transform_image)
    print(x_of_point3_of_transform_image, y_of_point3_of_transform_image)

    x = (x_of_point3_of_transform_image + x_of_point1_of_transform_image) / 2
    y = (y_of_point3_of_transform_image + y_of_point1_of_transform_image) / 2

draw_params = dict(matchColor=(0, 255, 0),
                   singlePointColor=(255, 0, 0),
                   matchesMask=matchesMask,
                   flags=0)

output_image = cv2.drawKeypoints(img2, kp1, 0, (0, 0, 255),
                                 flags=cv2.DRAW_MATCHES_FLAGS_NOT_DRAW_SINGLE_POINTS)

# displaying the image with keypoints as the
# output on the screen

plt.imshow(output_image)

# plotting image
plt.show()

img3 = cv2.drawMatchesKnn(img1, kp1, img2, kp2, matches, None, **draw_params)
cv2.imshow("绘制矩形", img3)
cv2.waitKey(100000)


#
# window_name = 'Image'
#
# # Center coordinates
# center_coordinates = (int(x), int(y))
# print(center_coordinates)
#
# # Radius of circle
# radius = 20
#
# # Blue color in BGR
# color = (0, 0, 0)
#
# # Line thickness of 2 px
# thickness = 2
#
# print(x_of_point1_of_transform_image)
# # Using cv2.circle() method
# # Draw a circle with blue line borders of thickness of 2 px
# cv2.circle(img2, center_coordinates, 10, (255, 0, 0), 2)
# cv2.rectangle(img2, (x_of_point1_of_transform_image, y_of_point1_of_transform_image),
#               (x_of_point3_of_transform_image, y_of_point3_of_transform_image), (255, 0, 0), 2)  # 画大矩形
# image = cv2.cvtColor(img2, cv2.COLOR_BGR2RGB)  # 色彩空间转换
# cv2.imshow("绘制矩形", image)
# cv2.waitKey(100000)



import re

from makima.openCV.kmeans_run import kmeans_run
from .stack import Stack
import time

from ..image_object import ImageObject


def wait_function(timeout, use_re, func, *args, **query):
    time_started_sec = time.time()
    query_string = list(query.values())[0]
    query_method = list(query.keys())[0]
    while time.time() < time_started_sec + timeout / 1000.0:
        result = func(use_re, *args, **query)
        if result is not None:
            finish_time = time.time() - time_started_sec
            # print("Found element in {} s".format(finish_time))
            return result
    error = "Can't find element/elements in %s s by %s = %s" % (timeout / 1000.0, query_method, query_string)
    raise TimeoutError(error)


def find_element_by_image(path, distance, **method):
    query_method = list(method.items().mapping.keys())[0]
    query_string = list(method.items().mapping.values())[0]
    x, y = kmeans_run(path, distance)
    if x is not None:
        return ImageObject(x, y)
    else:
        return None


def find_element_by_query(use_re, *args, **query):
    result_list = []
    all_node = Stack()
    all_node.push(args[0])
    result = None

    while all_node.is_not_empty():
        element = all_node.pop()
        elements_list = element.get_acc_children_elements()

        if use_re:
            result_list = [eval("element.get_{}".format(query_method), {}, {"element": element}) for
                           query_method, query_string in query.items() if
                           re.match(str(query_string), str(eval("element.get_{}".format(query_method), {}, {"element":element}))) is not None]

        else:
            result_list = [eval("element.get_{}".format(query_method), {}, {"element": element}) for
                           query_method, query_string in query.items() if
                           eval("element.get_{}".format(query_method), {}, {"element": element})
                           == query_string]
        if len(query) == len(result_list):
            if len(args) > 1 and "last" in args[1]:
                result = all_node.pop()
            elif len(args) > 1 and "next" in args[1]:
                result = last_element
            else:
                result = element

        last_element = element
        if len(elements_list) > 0:
            for child_element in elements_list:
                all_node.push(child_element)
    return result


def find_elements_by_query(use_re, *args, **query):
    all_node = Stack()
    all_node.push(args[0])
    result = None

    while all_node.is_not_empty():
        element = all_node.pop()
        elements_list = element.get_acc_children_elements()

        if use_re:
            result_list = [eval("element.get_{}".format(query_method), {}, {"element": element}) for
                           query_method, query_string in query.items() if
                           re.match(str(query_string), str(eval("element.get_{}".format(query_method), {},
                                                                {"element": element}))) is not None]
        else:
            result_list = [eval("element.get_{}".format(query_method), {}, {"element": element}) for
                           query_method, query_string in query.items() if
                           eval("element.get_{}".format(query_method), {}, {"element": element})
                           == query_string]

        if len(query) == len(result_list):
            result.append(element)

        if len(elements_list) > 0:
            for child_element in elements_list:
                all_node.push(child_element)
    return result



import cv2
import numpy as np
import pyscreeze


def kmeans_run(path, distance=0.4):
    img = pyscreeze.screenshot()
    img1 = np.array(img)
    print(img1)
    img2 = cv2.imread(path,
                      0)
    # 初始化SIFT选择器
    sift = cv2.SIFT_create()

    # 检测关键点并计算描述符
    kp1, des1 = sift.detectAndCompute(img1, None)
    kp2, des2 = sift.detectAndCompute(img2, None)

    # FLANN 参数
    FLANN_INDEX_KDTREE = 0
    index_params = dict(algorithm=FLANN_INDEX_KDTREE, trees=5)
    search_params = dict(checks=50)

    flann = cv2.FlannBasedMatcher(index_params, search_params)

    # 使用基于FLANN的匹配器, 筛选符合条件的坐标
    matches = flann.knnMatch(des1, des2, k=2)

    good = []
    for i, (m, n) in enumerate(matches):
        # distance越小匹配度越高
        if m.distance < distance * n.distance:
            good.append(m)

    if len(good) > 10:
        # Gets the coordinates of the key points
        src_pts = np.float32([kp1[m.queryIdx].pt for m in good]).reshape(-1, 1, 2)
        dst_pts = np.float32([kp2[m.trainIdx].pt for m in good]).reshape(-1, 1, 2)
        # Compute the transformation matrix and MASK
        M, mask = cv2.findHomography(src_pts, dst_pts, cv2.RANSAC, 5.0)
        h, w = img1.shape

        # Use the transformation matrix to obtain the coordinates of the four corners of the original image after
        # transformation
        pts = np.float32([[0, 0], [0, h - 1], [w - 1, h - 1], [w - 1, 0]]).reshape(-1, 1, 2)
        dst = cv2.perspectiveTransform(pts, M)

        # Get the coordinates of the lower right corner of the transformed image
        x_of_point1_of_transform_image = [np.int32(dst)][0][0][0][0]
        y_of_point1_of_transform_image = [np.int32(dst)][0][0][0][1]

        # Get the coordinates of the lower right corner of the transformed image
        x_of_point3_of_transform_image = [np.int32(dst)][0][2][0][0]
        y_of_point3_of_transform_image = [np.int32(dst)][0][2][0][1]

        x = (x_of_point3_of_transform_image + x_of_point1_of_transform_image) / 2
        y = (y_of_point3_of_transform_image + y_of_point1_of_transform_image) / 2
        return x, y
    else:
        return None, None



import ctypes
import re
from ctypes import *
from ctypes import wintypes
from ctypes.wintypes import HWND, CHAR, LPSTR



def convert_wildcard_to_regex(wildcard):
    """
    Converts wildcard to regex.

    :param str wildcard: wildcard.
    :rtype: str
    :return: regex pattern.
    """
    regex = re.escape(wildcard)
    regex = regex.replace(r'\?', r'[\s\S]{1}')
    regex = regex.replace(r'\*', r'[\s\S]*')

    return '^%s$' % regex


def replace_inappropriate_symbols(text):
    """
    Replaces inappropriate symbols e.g. \xa0 (non-breaking space) to
    normal space.

    :param str text: text in which symbols should be replaced.
    :rtype: str
    :return: processed text.
    """
    replace_pairs = [(u'\xa0', ' '),
                     (u'\u2014', '-')]

    for from_, to_ in replace_pairs:
        text = text.replace(from_, to_)

    return text


def verify_xy_coordinates(x, y):
    """
    Verifies that x and y is instance of int otherwise raises exception.

    :param x: x variable.
    :param y: y variable.
    """
    if not isinstance(x, (int, float)) or not isinstance(y, (int, float)):
        raise Exception(
            'x and y arguments should hold int coordinates.')


def verify_mouse_button_name(button_name, supported_names):
    """
    Verifies that button name is supported otherwise raises exception.

    :param str button_name: button name.
    :param list[str] supported_names: supported button names.
    """
    if button_name not in supported_names:
        raise Exception(
            'Button name should be one of supported %s.' %
            repr(supported_names))


def get_window_title(hwnd):
    """Get native window title."""
    cb = windll.user32.GetWindowTextLengthW(hwnd) + 1
    title = create_unicode_buffer(cb)
    windll.user32.GetWindowTextW(hwnd, title, cb)
    return title.value


def get_window_class_name(hwnd):
    _GetClassNameA = windll.user32.GetClassNameA
    _GetClassNameA.argtypes = [HWND, LPSTR, ctypes.c_int]
    _GetClassNameA.restype = ctypes.c_int

    nMaxCount = 0x1000
    dwCharSize = sizeof(CHAR)
    while 1:
        lpClassName = create_string_buffer(nMaxCount)
        nCount = _GetClassNameA(hwnd, lpClassName, nMaxCount)
        if nCount == 0:
            raise WinError()
        if nCount < nMaxCount - dwCharSize:
            break
        nMaxCount += 0x1000
    return str(lpClassName.value)


def find_windows(name, class_name=None):
    """
    Windows Platform find windows
    :param name is windows name and class_name is unique one
    return hwnd list
    """
    WNDENUMPROC = WINFUNCTYPE(wintypes.BOOL, wintypes.HWND, wintypes.LPARAM)
    hwnd_list = []

    def EnumWindowsProc(hwnd, lParam):
        length = user32.GetWindowTextLengthW(hwnd) + 1
        buffer = create_unicode_buffer(length)
        user32.GetWindowTextW(hwnd, buffer, length)
        if buffer.value == name and user32.IsWindowVisible(hwnd):
            if class_name is not None:
                if "b'%s'" % class_name == get_window_class_name(hwnd):
                    hwnd_list.append(hwnd)
            else:
                hwnd_list.append(hwnd)
        return True

    user32 = windll.LoadLibrary("user32.dll")
    user32.EnumWindows(WNDENUMPROC(EnumWindowsProc), 0)
    return hwnd_list


def set_focus_window(hwnd):
    """Set windows front to desktop"""
    user32 = windll.LoadLibrary("user32.dll")
    if user32.IsIconic(hwnd):
        user32.ShowWindow(hwnd, 9)
    user32.SetForegroundWindow(hwnd)


def show_windows(hwnd, cmd_show):
    """ Maxinum or Mininum windows
        SW_HIDE = 0
        SW_SHOWNORMAL = 1
        SW_SHOWMINIMIZED = 2
        SW_SHOWMAXIMIZED = 3
        SW_SHOWNOACTIVATE = 4
        SW_SHOW = 5
        SW_MINIMIZE = 6
        SW_SHOWMINNOACTIVE = 7
        SW_SHOWNA = 8
        SW_RESTORE = 9
        SW_SHOWDEFAULT = 10
    """

    user32 = windll.LoadLibrary("user32.dll")
    user32.ShowWindow(hwnd, cmd_show)



#!/usr/bin/env python
from collections import deque
from xml.dom import minidom

import comtypes.client

from makima.windows.utils.find_ui_element import wait_function, find_element_by_query, find_elements_by_query
from makima.windows.utils.mouse import WinMouse
import comtypes.client


CO_E_OBJNOTCONNECTED = -2147220995
UIAutomationCore = comtypes.client.GetModule("UIAutomationCore.dll")
IUIAutomation = comtypes.client.CreateObject("{ff48dba4-60ef-4201-aa87-54103eef594e}",
                                             interface=UIAutomationCore.IUIAutomation)

_IUIAutomation = comtypes.CoCreateInstance(comtypes.gen.UIAutomationClient.CUIAutomation._reg_clsid_,
                                           interface=comtypes.gen.UIAutomationClient.IUIAutomation,
                                           clsctx=comtypes.CLSCTX_INPROC_SERVER)

UIAutomationClient = comtypes.gen.UIAutomationClient

_control_type = {
    UIAutomationClient.UIA_ButtonControlTypeId: 'UIA_ButtonControlTypeId',
    UIAutomationClient.UIA_CalendarControlTypeId: 'UIA_CalendarControlTypeId',
    UIAutomationClient.UIA_CheckBoxControlTypeId: 'UIA_CheckBoxControlTypeId',
    UIAutomationClient.UIA_ComboBoxControlTypeId: 'UIA_ComboBoxControlTypeId',
    UIAutomationClient.UIA_CustomControlTypeId: 'UIA_CustomControlTypeId',
    UIAutomationClient.UIA_DataGridControlTypeId: 'UIA_DataGridControlTypeId',
    UIAutomationClient.UIA_DataItemControlTypeId: 'UIA_DataItemControlTypeId',
    UIAutomationClient.UIA_DocumentControlTypeId: 'UIA_DocumentControlTypeId',
    UIAutomationClient.UIA_EditControlTypeId: 'UIA_EditControlTypeId',
    UIAutomationClient.UIA_GroupControlTypeId: 'UIA_GroupControlTypeId',
    UIAutomationClient.UIA_HeaderControlTypeId: 'UIA_HeaderControlTypeId',
    UIAutomationClient.UIA_HeaderItemControlTypeId: 'UIA_HeaderItemControlTypeId',
    UIAutomationClient.UIA_HyperlinkControlTypeId: 'UIA_HyperlinkControlTypeId',
    UIAutomationClient.UIA_ImageControlTypeId: 'UIA_ImageControlTypeId',
    UIAutomationClient.UIA_ListControlTypeId: 'UIA_ListControlTypeId',
    UIAutomationClient.UIA_ListItemControlTypeId: 'UIA_ListItemControlTypeId',
    UIAutomationClient.UIA_MenuBarControlTypeId: 'UIA_MenuBarControlTypeId',
    UIAutomationClient.UIA_MenuControlTypeId: 'UIA_MenuControlTypeId',
    UIAutomationClient.UIA_MenuItemControlTypeId: 'UIA_MenuItemControlTypeId',
    UIAutomationClient.UIA_PaneControlTypeId: 'UIA_PaneControlTypeId',
    UIAutomationClient.UIA_ProgressBarControlTypeId: 'UIA_ProgressBarControlTypeId',
    UIAutomationClient.UIA_RadioButtonControlTypeId: 'UIA_RadioButtonControlTypeId',
    UIAutomationClient.UIA_ScrollBarControlTypeId: 'UIA_ScrollBarControlTypeId',
    UIAutomationClient.UIA_SeparatorControlTypeId: 'UIA_SeparatorControlTypeId',
    UIAutomationClient.UIA_SliderControlTypeId: 'UIA_SliderControlTypeId',
    UIAutomationClient.UIA_SpinnerControlTypeId: 'UIA_SpinnerControlTypeId',
    UIAutomationClient.UIA_SplitButtonControlTypeId: 'UIA_SplitButtonControlTypeId',
    UIAutomationClient.UIA_StatusBarControlTypeId: 'UIA_StatusBarControlTypeId',
    UIAutomationClient.UIA_TabControlTypeId: 'UIA_TabControlTypeId',
    UIAutomationClient.UIA_TabItemControlTypeId: 'UIA_TabItemControlTypeId',
    UIAutomationClient.UIA_TableControlTypeId: 'UIA_TableControlTypeId',
    UIAutomationClient.UIA_TextControlTypeId: 'UIA_TextControlTypeId',
    UIAutomationClient.UIA_ThumbControlTypeId: 'UIA_ThumbControlTypeId',
    UIAutomationClient.UIA_TitleBarControlTypeId: 'UIA_TitleBarControlTypeId',
    UIAutomationClient.UIA_ToolBarControlTypeId: 'UIA_ToolBarControlTypeId',
    UIAutomationClient.UIA_ToolTipControlTypeId: 'UIA_ToolTipControlTypeId',
    UIAutomationClient.UIA_TreeControlTypeId: 'UIA_TreeControlTypeId',
    UIAutomationClient.UIA_TreeItemControlTypeId: 'UIA_TreeItemControlTypeId',
    UIAutomationClient.UIA_WindowControlTypeId: 'UIA_WindowControlTypeId'
}

_tree_scope = {
    'ancestors': UIAutomationClient.TreeScope_Ancestors,
    'children': UIAutomationClient.TreeScope_Children,
    'descendants': UIAutomationClient.TreeScope_Descendants,
    'element': UIAutomationClient.TreeScope_Element,
    'parent': UIAutomationClient.TreeScope_Parent,
    'subtree': UIAutomationClient.TreeScope_Subtree
}


class WinUIElement(object):
    def __init__(self, IUIAutomationElement):
        self.IUIAutomationElement = IUIAutomationElement

    _mouse = WinMouse()

    @property
    def get_toggle_state(self):
        """Retrieves the UI Automation element value
        :rtype : unicode
        """
        IUnknown = self.IUIAutomationElement.GetCurrentPattern(UIAutomationClient.UIA_TogglePatternId)
        IUIAutomationTogglePattern = IUnknown.QueryInterface(UIAutomationClient.IUIAutomationTogglePattern)
        return IUIAutomationTogglePattern.CurrentToggleState

    CurrentToggleState = get_toggle_state

    @property
    def get_acc_value(self):
        """Retrieves the UI Automation element value
        :rtype : unicode
        """
        IUnknown = self.IUIAutomationElement.GetCurrentPattern(UIAutomationClient.UIA_TextPatternId)
        IUIAutomationTextPattern = IUnknown.QueryInterface(UIAutomationClient.IUIAutomationTextPattern)
        return IUIAutomationTextPattern.DocumentRange.getText(-1)

    CurrentValue = get_acc_value

    @property
    def get_automation_id(self):
        """Retrieves the UI Automation identifier of the element
        :rtype : unicode
        """
        return self.IUIAutomationElement.CurrentAutomationId

    AutomationId = get_automation_id

    @property
    def get_acc_location(self):
        """Retrieves the coordinates of the rectangle that completely encloses the element.
        Returns tuple (left, top, right, bottom)
        :rtype : tuple
        """
        rect = self.IUIAutomationElement.CurrentBoundingRectangle
        return rect.left, rect.top, rect.right, rect.bottom

    BoundingRectangle = get_acc_location

    @property
    def get_class_name(self):
        """Retrieves the class name of the element
        :rtype : unicode
        """
        return self.IUIAutomationElement.CurrentClassName

    ClassName = get_class_name

    @property
    def get_control_type(self):
        """Retrieves the control type of the element
        :rtype : int
        """
        return self.IUIAutomationElement.CurrentControlType

    ControlType = get_control_type

    @property
    def get_control_type_name(self):
        """Retrieves the name of the control type of the element.
        Ref: http://msdn.microsoft.com/en-us/library/windows/desktop/ee671198(v=vs.85).aspx
        :rtype : str
        """
        return _control_type[self.get_control_type]

    ControlTypeName = get_control_type_name

    @property
    def get_is_enabled(self):
        """Indicates whether the element is enabled
        :rtype : bool
        """
        return bool(self.IUIAutomationElement.CurrentIsEnabled)

    IsEnabled = get_is_enabled

    @property
    def get_acc_name(self):
        """Retrieves the name of the element
        :rtype : unicode
        """
        return self.IUIAutomationElement.CurrentName

    Name = get_acc_name

    @property
    def get_default_action(self):
        IUnknown = self.IUIAutomationElement.GetCurrentPattern(UIAutomationClient.UIA_LegacyIAccessiblePatternId)
        IUIAutomationLegacyIAccessiblePattern = IUnknown.QueryInterface(
            UIAutomationClient.IUIAutomationLegacyIAccessiblePattern)
        return str(IUIAutomationLegacyIAccessiblePattern.CurrentDefaultAction)

    DefaultAction = get_default_action

    def set_value(self, value):
        """Retrieves the UI Automation element value
        :rtype : unicode
        """
        IUnknown = self.IUIAutomationElement.GetCurrentPattern(UIAutomationClient.UIA_ValuePatternId)
        IUIAutomationValuePattern = IUnknown.QueryInterface(UIAutomationClient.IUIAutomationValuePattern)
        IUIAutomationValuePattern.SetValue(value)

    def get_clickable_point(self):
        """Retrieves a point on the element that can be clicked
        Returns tuple (x, y) if a clickable point was retrieved, or None otherwise
        :rtype : tuple
        """
        point = self.IUIAutomationElement.GetClickablePoint()
        if point[1]:
            return point[0].x, point[0].y
        else:
            return None

    def _build_condition(self, Name, ControlType, AutomationId):
        condition = _IUIAutomation.CreateTrueCondition()

        if Name is not None:
            name_condition = _IUIAutomation.CreatePropertyCondition(UIAutomationClient.UIA_NamePropertyId, Name)
            condition = _IUIAutomation.CreateAndCondition(condition, name_condition)

        if ControlType is not None:
            control_type_condition = _IUIAutomation.CreatePropertyCondition(
                UIAutomationClient.UIA_ControlTypePropertyId, ControlType)
            condition = _IUIAutomation.CreateAndCondition(condition, control_type_condition)

        if AutomationId is not None:
            automation_id_condition = _IUIAutomation.CreatePropertyCondition(
                UIAutomationClient.UIA_AutomationIdPropertyId, AutomationId)
            condition = _IUIAutomation.CreateAndCondition(condition, automation_id_condition)

        return condition

    def findfirst(self, tree_scope, Name=None, ControlType=None, AutomationId=None):
        """Retrieves the first child or descendant element that matches specified conditions
        Returns None if there is no element that matches specified conditions
        If Name is None, element with any name will match
        If ControlType is None, element with any control type will match
        :param tree_scope: Should be one of 'element', 'children', 'descendants', 'parent', 'ancestors', 'subtree'.
            Ref: http://msdn.microsoft.com/en-us/library/windows/desktop/ee671699(v=vs.85).aspx
        :type tree_scope: str
        :param Name: Name of the element.
        :type Name: str
        :param ControlType: Control type of the element (one of UIAutomationClient.UIA_*ControlTypeId).
            Ref: http://msdn.microsoft.com/en-us/library/windows/desktop/ee671198(v=vs.85).aspx
        :type ControlType: int
        :param AutomationId: UI Automation identifier (ID) for the automation element.
            Ref: http://msdn.microsoft.com/en-us/library/windows/desktop/ee695997(v=vs.85).aspx
        :type AutomationId: str
        :rtype : _UIAutomationElement
        """
        tree_scope = _tree_scope[tree_scope]
        condition = self._build_condition(Name, ControlType, AutomationId)
        element = self.IUIAutomationElement.FindFirst(tree_scope, condition)
        return WinUIElement(element) if element else None

    #
    def findall(self, tree_scope, Name=None, ControlType=None, AutomationId=None):
        """Returns list of UI Automation elements that satisfy specified conditions
        Returns empty list if there are no elements that matches specified conditions
        If Name is None, elements with any name will match
        If ControlType is None, elements with any control type will match
        :param tree_scope: Should be one of 'element', 'children', 'descendants', 'parent', 'ancestors', 'subtree'.
            Ref: http://msdn.microsoft.com/en-us/library/windows/desktop/ee671699(v=vs.85).aspx
        :type tree_scope: str
        :param Name: Name of the element.
        :type Name: str
        :param ControlType: Control type of the element (one of UIAutomationClient.UIA_*ControlTypeId).
            Ref: http://msdn.microsoft.com/en-us/library/windows/desktop/ee671198(v=vs.85).aspx
        :type ControlType: int
        :param AutomationId: UI Automation identifier (ID) for the automation element.
            Ref: http://msdn.microsoft.com/en-us/library/windows/desktop/ee695997(v=vs.85).aspx
        :type AutomationId: str
        :rtype : list
        """
        tree_scope = _tree_scope[tree_scope]
        condition = self._build_condition(Name, ControlType, AutomationId)

        IUIAutomationElementArray = self.IUIAutomationElement.FindAll(tree_scope, condition)
        return [WinUIElement(IUIAutomationElementArray.GetElement(i)) for i in
                range(IUIAutomationElementArray.Length)]

    def Invoke(self):
        IUnknown = self.IUIAutomationElement.GetCurrentPattern(UIAutomationClient.UIA_InvokePatternId)
        IUIAutomationInvokePattern = IUnknown.QueryInterface(UIAutomationClient.IUIAutomationInvokePattern)
        IUIAutomationInvokePattern.Invoke()

    def __str__(self):
        return '<%s (Name: %s, Class: %s, AutomationId: %s>' % (
            self.get_control_type_name, self.get_acc_name, self.get_class_name, self.get_automation_id)

    def toxml(self):
        xml = minidom.Document()
        queue = deque()
        queue.append((self, xml))
        while queue:
            element, xml_node = queue.popleft()
            xml_element = minidom.Element(element.CurrentControlTypeName)
            xml_element.setAttribute('Name', str(element.CurrentName))
            xml_element.setAttribute('AutomationId', str(element.CurrentAutomationId))
            xml_element.setAttribute('ClassName', str(element.CurrentClassName))
            xml_element.ownerDocument = xml
            xml_node.appendChild(xml_element)
            for child in element.findall('children'):
                queue.append((child, xml_element))
        return xml.toprettyxml()

    def get_acc_children_elements(self):
        return self.findall("children")

    '''
    query:
        automation_id=automation id
        acc_name=acc name
        acc_value=acc value
        class_name=class name
        control_type_name=control type name
    '''

    def find_element_by_wait(self, timeout=5000, use_re=False, **query):
        return wait_function(timeout, use_re, find_element_by_query, self, **query)

    def find_next_element_by_wait(self, timeout=5000, use_re=False, **query):
        return wait_function(timeout, use_re, find_element_by_query, self, "next", **query)

    def find_last_element_by_wait(self, timeout=5000, use_re=False, **query):
        return wait_function(timeout, use_re, find_element_by_query, self, "last", **query)

    def find_elements_by_wait(self, timeout=5000, use_re=False, **query):
        return wait_function(timeout, use_re, find_elements_by_query, self, **query)

    def click(self, x_offset=None, y_offset=None):
        if x_offset is not None:
            x = x_offset
            y = y_offset
        else:
            x, y = self.get_clickable_point()
        self._mouse.click(x, y)

    def right_click(self, x_offset=None, y_offset=None):
        if x_offset is not None:
            x = x_offset
            y = y_offset
        else:
            x, y = self.get_clickable_point()
        self._mouse.click(x, y, self._mouse.RIGHT_BUTTON)

    def double_click(self, x_offset=None, y_offset=None):
        if x_offset is not None:
            x = x_offset
            y = y_offset
        else:
            x, y = self.get_clickable_point()
        self._mouse.double_click(x, y)

    def drag_to(self, x2, y2, x_offset=None, y_offset=None, smooth=True):
        if x_offset is not None:
            x = x_offset
            y = y_offset
        else:
            x, y = self.get_clickable_point()
        self._mouse.drag(x, y, x2, y2, smooth)





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

def parse_requirements(filename):
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]

setuptools.setup(
    name="Makima",
    version="0.1.5",
    author="hank.huang",
    author_email="hank.huang550@gmail.com",
    description="A cross-platform desktop automated testing framework.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/letmeNo1/Makima",
    packages=setuptools.find_packages(),
    install_requires=parse_requirements('requirements.txt'),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



#!/usr/bin/env python

import abc
import xml.dom.minidom
from abc import ABCMeta, abstractmethod

from comtypes import IUnknown


class IElement(IUnknown):
    """
    Class that describes UI object.
    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def click(self, x_offset=None, y_offset=None):
        """
        Clicks by left mouse button on this object.

        :param int x_offset: if not defined half of element width will be used.
        :param int y_offset: y offset, if not defined half of element height 
        will be used.
        """

    @abstractmethod
    def right_click(self, x_offset=None, y_offset=None):
        """
        Clicks by right mouse button on this object.

        :param int x_offset: if not defined half of element width will be used.
        :param int y_offset: y offset, if not defined half of element height 
        will be used.
        """

    @abstractmethod
    def double_click(self, x_offset=None, y_offset=None):
        """
        Double clicks by left mouse button on this object.

        :param int x_offset: if not defined half of element width will be used.
        :param int y_offset: y offset, if not defined half of element height 
        will be used
        """

    @abstractmethod
    def drag_to(self, x, y, x_offset=None, y_offset=None, smooth=True):
        """
        Drags this object to coordinates.

        :param int x: x coordinate.
        :param int y: y coordinate.
        :param int x_offset: if not defined half of element width will be used.
        :param int y_offset: if not defined half of element height 
        will be used.
        :param bool smooth: indicates is it needed to simulate smooth movement.
        """

    @abc.abstractmethod
    def proc_id(self):
        """
        Indicates process id.
        """

    @abc.abstractmethod
    def is_top_level_window(self):
        """
        Indicates is top level window or not.
        """

    @abc.abstractmethod
    def is_selected(self):
        """
        Indicates selected state.
        """

    @abc.abstractmethod
    def is_checked(self):
        """
        Indicates checked state.
        """

    @abc.abstractmethod
    def is_visible(self):
        """
        Indicates visible state.
        """

    @abc.abstractmethod
    def is_enabled(self):
        """
        Indicates enabled state.
        """

    @abc.abstractmethod
    def acc_parent_count(self):
        """
        Property for parent child count.
        """

    @abc.abstractmethod
    def acc_child_count(self):
        """
        Property for element child count.
        """

    @abc.abstractmethod
    def acc_name(self):
        """
        Property for element name.
        Also need to specify setter for this property
        """

    @abstractmethod
    def set_focus(self):
        """
        Sets focus to element.
        """

    @abc.abstractmethod
    def acc_location(self):
        """
        Property for element location.
        """

    @abc.abstractmethod
    def acc_value(self):
        """
        Property for element value.
        Also need to specify setter for this property.
        """

    @abstractmethod
    def set_value(self, value):
        """
        Sets element value.

        :param str value: element value.
        """

    @abc.abstractmethod
    def acc_description(self):
        """
        Property for element description.
        """

    @abc.abstractmethod
    def acc_parent(self):
        """
        Property for element parent.
        """

    @abc.abstractmethod
    def acc_selection(self):
        """
        Property for element selection.
        """

    @abc.abstractmethod
    def acc_focused_element(self):
        """
        Property for element in focus.
        """

    @abc.abstractmethod
    def acc_role_name(self):
        """
        Property for element role name.
        """

    @abstractmethod
    def __iter__(self):
        """Iterate all child Element"""

    @abstractmethod
    def is_object_exists(self, **kwargs):
        """
        Verifies is object exists.

        :param bool only_visible: flag that indicates will we search only
        through visible elements.
        :param str role: string or lambda e.g. lambda x: x == 13
        :param str name: string or lambda.
        :param str c_name: string or lambda.
        :param str location: string or lambda.
        :param str value: string or lambda.
        :param str description: string or lambda.
        :param str selection: string or lambda.
        :param str role_name: string or lambda.
        :param str parent_count: string or lambda.
        :param str child_count: string or lambda.
        :rtype: bool
        :return: True if object exists otherwise False.
        """

    def toxml(self):
        """
        Convert Element Tree to XML.
        """
        obj_document = xml.dom.minidom.Document()
        lst_queue = [(self, obj_document)]

        while lst_queue:
            obj_element, obj_tree = lst_queue.pop(0)
            role_name = obj_element.acc_role_name
            obj_name = obj_element.acc_name
            str_name = str(obj_name) if obj_name else ''
            str_location = ','.join(str(x) for x in obj_element.acc_location)
            obj_sub_tree = xml.dom.minidom.Element(role_name)
            obj_sub_tree.ownerDocument = obj_document

            try:
                obj_sub_tree.attributes['Name'] = str_name
            except:
                obj_sub_tree.attributes['Name'] = \
                    str_name.encode('unicode-escape')

            obj_sub_tree.attributes['Location'] = str_location
            obj_tree.appendChild(obj_sub_tree)

            if obj_element.acc_child_count:
                for obj_element_child in obj_element:
                    lst_queue.append((obj_element_child, obj_sub_tree))

        return obj_document.toprettyxml()



from makima.windows.utils.mouse import WinMouse


class ImageObject:
    def __init__(self, x=0, y=0):
        self.x = x
        self.y = y

    _mouse = WinMouse()

    def click(self):
        print(self.x)
        self._mouse.click(self.x, self.y)

    def right_click(self):
        self._mouse.click(self.x, self.y, self._mouse.RIGHT_BUTTON)

    def double_click(self):
        self._mouse.double_click(self.x, self.y)

    def drag_to(self, x2, y2, smooth=True):
        self._mouse.drag(self.x, self.y, x2, y2, smooth)



import ctypes
import ctypes.wintypes
from time import sleep

from makima.windows.utils import common
from makima.windows.call_win_api.i_mouse import IMouse


class WinMouse(IMouse):
    _MOUSEEVENTF_MOVE = 0x0001  # mouse move
    _MOUSEEVENTF_LEFTDOWN = 0x0002  # left button down
    _MOUSEEVENTF_LEFTUP = 0x0004  # left button up
    _MOUSEEVENTF_RIGHTDOWN = 0x0008  # right button down
    _MOUSEEVENTF_RIGHTUP = 0x0010  # right button up
    _MOUSEEVENTF_MIDDLEDOWN = 0x0020  # middle button down
    _MOUSEEVENTF_MIDDLEUP = 0x0040  # middle button up
    _MOUSEEVENTF_ABSOLUTE = 0x8000  # absolute move
    _MOUSEEVENTF_XDOWN = 0x0080  # X button down
    _MOUSEEVENTF_XUP = 0x0100  # X button up
    _MOUSEEVENTF_WHEEL = 0x0800  # wheel button is rotated
    _MOUSEEVENTF_HWHEEL = 0x01000  # wheel button is tilted

    _SM_CXSCREEN = 0
    _SM_CYSCREEN = 1

    LEFT_BUTTON = u'b1c'
    RIGHT_BUTTON = u'b3c'
    _SUPPORTED_BUTTON_NAMES = [LEFT_BUTTON, RIGHT_BUTTON]

    def _compose_mouse_event(self, name, press=True, release=False):
        """
        Composes mouse event based on button name and action flags.

        :param str name: mouse button name. Should be one
        of: 'b1c' - left button or 'b3c' - right button.
        :param bool press: flag indicating whether event should indicate 
        button press.
        :param bool release: flag indicating whether event should indicate
        button release.
        """
        mouse_event = 0
        if name == self.LEFT_BUTTON:
            if press:
                mouse_event += self._MOUSEEVENTF_LEFTDOWN
            if release:
                mouse_event += self._MOUSEEVENTF_LEFTUP
        if name == self.RIGHT_BUTTON:
            if press:
                mouse_event += self._MOUSEEVENTF_RIGHTDOWN
            if release:
                mouse_event += self._MOUSEEVENTF_RIGHTUP

        return mouse_event

    def _do_event(self, flags, x, y, data, extra_info):
        """
        Generates mouse event fo a special coordinate.

        :param int flags: integer value holding flags that describes mouse 
        events to trigger. Can be a combination of:
                _MOUSEEVENTF_MOVE = 0x0001 # mouse move
                _MOUSEEVENTF_LEFTDOWN = 0x0002 # left button down
                _MOUSEEVENTF_LEFTUP = 0x0004 # left button up
                _MOUSEEVENTF_RIGHTDOWN = 0x0008 # right button down
                _MOUSEEVENTF_RIGHTUP = 0x0010 # right button up
                _MOUSEEVENTF_MIDDLEDOWN = 0x0020 # middle button down
                _MOUSEEVENTF_MIDDLEUP = 0x0040 # middle button up
                _MOUSEEVENTF_ABSOLUTE = 0x8000 # absolute move
                _MOUSEEVENTF_XDOWN = 0x0080 # X button down
                _MOUSEEVENTF_XUP = 0x0100 # X button up
                _MOUSEEVENTF_WHEEL = 0x0800 # wheel button is rotated
                _MOUSEEVENTF_HWHEEL = 0x01000 # wheel button is tilted
        :param int x: x coordinate.
        :param int y: y coordinate.
        :param int data: value holding additional event data, for ex.:
              * If flags contains _MOUSEEVENTF_WHEEL, then data specifies the
                amount of wheel movement. A positive value indicates that the
                wheel was rotated forward, away from the user; a negative
                value indicates that the wheel was rotated backward, toward
                the user. One wheel click is defined as WHEEL_DELTA, which is
                120.
              * If flags contains _MOUSEEVENTF_HWHEEL, then data specifies the
                amount of wheel movement. A positive value indicates that the
                wheel was tilted to the right; a negative value indicates that
                the wheel was tilted to the left.
              * If flags contains _MOUSEEVENTF_XDOWN or _MOUSEEVENTF_XUP, then
                data specifies which X buttons were pressed or released. This
                value may be any combination of the following flags.
              * If flags is not _MOUSEEVENTF_WHEEL, _MOUSEEVENTF_XDOWN, or
                _MOUSEEVENTF_XUP, then data should be zero.
        :param int extra_info: value with additional value associated with
        the mouse event.
        """
        x_metric = ctypes.windll.user32.GetSystemMetrics(self._SM_CXSCREEN)
        y_metric = ctypes.windll.user32.GetSystemMetrics(self._SM_CYSCREEN)
        x_calc = 65536 * x / x_metric + 1
        y_calc = 65536 * y / y_metric + 1
        ctypes.windll.user32.mouse_event(
            flags, int(x_calc), int(y_calc), data, extra_info)

    def move(self, x, y, smooth=True):
        common.verify_xy_coordinates(x, y)

        old_x, old_y = self.get_position()

        for i in range(100):
            intermediate_x = old_x + (x - old_x) * (i + 1) / 100.0
            intermediate_y = old_y + (y - old_y) * (i + 1) / 100.0
            smooth and sleep(.01)

            self._do_event(self._MOUSEEVENTF_MOVE + self._MOUSEEVENTF_ABSOLUTE,
                           int(intermediate_x), int(intermediate_y), 0, 0)

    def drag(self, x1, y1, x2, y2, smooth=True):
        common.verify_xy_coordinates(x1, y1)
        common.verify_xy_coordinates(x2, y2)

        self.press_button(x1, y1, self.LEFT_BUTTON)
        self.move(x2, y2, smooth=smooth)
        self.release_button(self.LEFT_BUTTON)

    def press_button(self, x, y, button_name=LEFT_BUTTON):
        common.verify_xy_coordinates(x, y)
        common.verify_mouse_button_name(button_name,
                                        self._SUPPORTED_BUTTON_NAMES)

        self.move(x, y)
        self._do_event(
            self._compose_mouse_event(button_name, press=True, release=False),
            0, 0, 0, 0)

    def release_button(self, button_name=LEFT_BUTTON):
        common.verify_mouse_button_name(button_name,
                                        self._SUPPORTED_BUTTON_NAMES)

        self._do_event(
            self._compose_mouse_event(button_name, press=False, release=True),
            0, 0, 0, 0)

    def click(self, x, y, button_name=LEFT_BUTTON):
        common.verify_xy_coordinates(x, y)
        common.verify_mouse_button_name(button_name,
                                        self._SUPPORTED_BUTTON_NAMES)
        self.move(x, y, False)
        self._do_event(
            self._compose_mouse_event(button_name, press=True, release=True),
            0, 0, 0, 0)

    def double_click(self, x, y, button_name=LEFT_BUTTON):
        common.verify_xy_coordinates(x, y)
        common.verify_mouse_button_name(button_name,
                                        self._SUPPORTED_BUTTON_NAMES)

        self.move(x, y)
        self._do_event(
            self._compose_mouse_event(button_name, press=True, release=True),
            0, 0, 0, 0)
        self._do_event(
            self._compose_mouse_event(button_name, press=True, release=True),
            0, 0, 0, 0)

    def wheel(self, x, y, movement):
        common.verify_xy_coordinates(x, y)
        self.move(x, y, False)
        self._do_event(self._MOUSEEVENTF_WHEEL, x, y, movement, 0)

    def get_position(self):
        obj_point = ctypes.wintypes.POINT()
        ctypes.windll.user32.GetCursorPos(ctypes.byref(obj_point))

        return obj_point.x, obj_point.y
